import { defineStore } from "pinia"
import { ref } from "vue"
import { request } from "@/scripts/request"
import { Preferences } from "@capacitor/preferences"
import { useRealms } from "@/stores/realms"
import { useChannels } from "@/stores/channels"
import { useFriends } from "@/stores/friends"

export interface Userinfo {
  isReady: boolean
  isLoggedIn: boolean
  displayName: string
  idSet: { [id: string]: number }
  data: any
}

const defaultUserinfo: Userinfo = {
  isReady: false,
  isLoggedIn: false,
  displayName: "Citizen",
  idSet: {},
  data: null
}

export async function getAtk() {
  return (await Preferences.get({ key: "identity.access_token" })).value
}

export async function getRtk() {
  return (await Preferences.get({ key: "identity.refresh_token" })).value
}

export async function getIdSet() {
  const value = (await Preferences.get({ key: "userinfo.id_set" })).value
  if (value == null) return null
  else return JSON.parse(value)
}

export async function setIdSet(data: any) {
  await Preferences.set({ key: "userinfo.id_set", value: JSON.stringify(data) })
}

export async function checkLoggedIn(): Promise<boolean> {
  return (await Preferences.get({ key: "identity.access_token" })).value != null
}

export async function signout() {
  await Preferences.remove({ key: "identity.access_token" })
  await Preferences.remove({ key: "identity.refresh_token" })
  await Preferences.remove({ key: "userinfo.id_set" })
}

export const useUserinfo = defineStore("userinfo", () => {
  const userinfoHooks = {
    after: [useRealms().list, useChannels().list, useChannels().connect, useFriends().list]
  }

  const userinfo = ref(defaultUserinfo)
  const isReady = ref(false)

  async function readProfiles() {
    if (!(await checkLoggedIn())) {
      isReady.value = true
    }

    const res = await request("identity", "/api/users/me", {
      headers: { Authorization: `Bearer ${await getAtk()}` }
    })
    if (res.status !== 200) {
      return
    }

    const data = await res.json()
    let idSet = await getIdSet()

    if (idSet == null) {
      const federationResp = await Promise.all([
        request("interactive", "/api/users/me", {
          headers: { Authorization: `Bearer ${await getAtk()}` }
        }),
        request("messaging", "/api/users/me", {
          headers: { Authorization: `Bearer ${await getAtk()}` }
        })
      ])
      await setIdSet({
        interactive: (await federationResp[0].json())["id"],
        messaging: (await federationResp[1].json())["id"]
      })
      idSet = await getIdSet()
    }

    userinfo.value = {
      isReady: true,
      isLoggedIn: true,
      displayName: data["nick"],
      idSet: idSet,
      data: data
    }

    userinfoHooks.after.forEach((call) => call())
  }

  return { userinfo, isReady, readProfiles }
})
